<?php
/**
 * Domyślny sterownik praw
 * 
 * @package MCM
 * @subpackage independent
 * @author Robert (nospor) Nodzewski (email: nospor at gmail dot com)
 * @copyright 2006-2008 Robert Nodzewski
 * @version 1.2
 * @access public
 */
class ShoutBoxDefaultRights implements ShoutBoxRights {
	
	/** W przykładzie prawa bede rozpoznawał na podstawie id i nicku */
	/** Oczywiście w waszych aplikacjach prawa rozpoznawajcie lepiej juz normalnie :) */
	public $Nick;
	public $Id;
	public $Logged;
	public $ModNick;//nick moderatora pobrany z pliku konfiguracyjnego
	function __construct($id, $nick){
		$this->Id = $id;
		$this->Nick = $nick;
		$this->Logged = !empty($_SESSION['nick']);
		global $cfg_shoutbox;
		$this->ModNick = is_string($cfg_shoutbox['mod_nick']) ? array($cfg_shoutbox['mod_nick']) : $cfg_shoutbox['mod_nick'];
	}
	
	/**
	 * Sprawdza, czy dane IP nie jest zbanowane
	 *
	 * @return mixed
	 *  - false - nie jest zbanowany
	 *  - true - jest zbanowany
	 *  - string - jest zbanowany. Dodatkowo zwrócony tekst jest tekstem do wyświetlenia
	 */
	public function IsBanned(){
		if (!file_exists('./sb_banned.php'))
			return false;
		$banned = include('./sb_banned.php');
		if (!is_array($banned)) //blad odczytu tablicy i nie mozna stwierdzić kto zbanowany
			return false;
		$ip = ShoutBox::GetIP();//pobieramy IP usera
		if (isset($banned['full'][$ip])) //jest zbanowany przez dokładne IP
			return empty($banned['full'][$ip]) ? 'Twoje IP jest zbanowane' : $banned['full'][$ip];
		if (!empty($banned['like'])){ //jeśli są podane IP ogólne, to sprawdzamy jeszcze w nich
			$ipArray = explode('.', $ip);
			foreach ($banned['like'] as $ipbanned => $text){
				$ipbannedArray = explode('.', $ipbanned);
				$ban = true;
				foreach ($ipbannedArray as $_key => $ipPart){
					if ($ipPart != '*' && $ipPart != $ipArray[$_key]){
						$ban = false;
						break;
					}
				}
				if ($ban)
					return empty($text) ? 'Twoje IP jest zbanowane' : $text;
			}
		}
		return false;
			
	}
	
	/**
	 * Funkcja określa czy dana osoba jest moderatorem
	 * Moderatorem jest zalogowana osoba o nicku określonym w pliku konfiguracyjnym
	 *
	 */
	public function IsModerator(){
		return $this->Logged && in_array($this->Nick,$this->ModNick);
	}
	
	public function CanEdit($message = null){
		if ($this->IsModerator()) //moderator moze edytowac wszystko
			return true;
		$banned = $this->IsBanned();
		if ($banned !== false)
			return $banned;	
		if ($message && !empty($this->Id) &&  $message['ID_USER']==$this->Id) //można edytowac wiadomość, ktorą się napisało
			return true;
		return 'Nie masz praw do edytowania tej wiadomości';
	}

	public function CanDelete($message = null) {
		if ($this->IsModerator()) //moderator moze usuwac wszystko
			return true;
		$banned = $this->IsBanned();
		if ($banned !== false)
			return $banned;	
		if ($message && !empty($this->Id) && $message['ID_USER']==$this->Id) //można usunąć wiadomość, ktorą się napisało
			return true;
		return 'Nie masz praw do usunięcia tej wiadomości';
	}

	public function CanAdd() {
		if ($this->IsModerator())
			return true;
		$banned = $this->IsBanned();
		if ($banned !== false)
			return $banned;	
		if (empty($this->Nick)) //jak nie podalismy nicka to nie mozemy pisac
			return false;
		elseif ($this->Nick == 'blocked') //w tym przykładzie koles o nicku "blocked" nie może nic pisać
			return 'Nie możesz dodawać wiadomości';
		if ($this->Logged) //osoby zalogowane mogą pisać
			return true;
		global $cfg_shoutbox; //mało eleganckie rozwiązanie, ale nie chce mi się nic wymyślać na poczet tego przykładu
		//jesli w konfigu okreslimy, ze gosc moze czytac i pisac, to pozwalamy tu
		if ($cfg_shoutbox['guest_read'] && $cfg_shoutbox['guest_write'])
			return true;	
		return false;
	}

	public function CanRead() {
		if ($this->IsModerator())
			return true;
		$banned = $this->IsBanned();
		if ($banned !== false)
			return $banned;	
		
		if ($this->Logged) //zalogowany może czytać
			return true;
		
		global $cfg_shoutbox; //mało eleganckie rozwiązanie, ale nie chce mi się nic wymyślać na poczet tego przykładu
		if ($cfg_shoutbox['guest_read']) //jesli w konfigu określimy ze gość moze czytać, to moze :)
			return true;
		return false;	
	}

	public function CanReadHistory() {
		if ($this->IsModerator()) //moderator może przeglądać historię do bólu :)
			return true;	
		$banned = $this->IsBanned();
		if ($banned !== false)
			return $banned;	
		if (!$this->Logged) //goście nie mogą czytać (tak sobie wymyslilem. zmiencie jak chcecie)
			return false;
		//pozostali użytkownicy tylko 2 razy mogą się cofnąć (tak sobie wymyslilem. zmiencie jak chcecie)
		if ($_SESSION['history'] >= 2)
			return 'Tylko moderator może przeglądać całą historię';
		$_SESSION['history']++;	
		return true;
	}
	
	public function CanRunCommand($command){
		if ($this->IsModerator())
			return true;
		$banned = $this->IsBanned();
		if ($banned !== false)
			return $banned;	
		
		if ($this->Logged){
			if ($command == 'ip'){
				return $this->IsModerator() ? true : 'Tylko moderatorzy mogą wykonać tę komendę';
			}
			//pozostałe komendy zalogowani użytkownicy mogą wykonywać
			return true;
		}	
		//goscie moga wykonac tylko komende help	
		return $command == 'help' ? true : 'Tylko zalogowani użytkownicy mogą wykonać tę komendę';
	}
	
}
?>