<?php
/**
 * Komenda ip
 * Słuzy do zażądzania adresami IP
 * Wywolanie: 
 * /ip show nick - pokazuje IP uzytkownika o nicku nick
 * /ip shownick IP - pokazuje nicki dla danego ip
 * 
 * @package MCM
 * @subpackage independent
 * @author Robert (nospor) Nodzewski (email: nospor at gmail dot com)
 * @copyright 2006-2008 Robert Nodzewski
 * @access public
 */
class ShoutBoxCommandIp implements ShoutBoxCommand {
	public function Run($arg, ShoutBox $sbObject, ShoutBoxDB $dbObject, ShoutBoxFormatter $formatterObject, ShoutBoxRights $rightsObject){
		if (!($dbObject instanceof ShoutBoxDBMysql)){ //komenda IP wykonywana jest tylko dla sterownika mysql
			$sbObject->AddAdminMessage('Komenda IP nie jest dosŧępna dla aktualnie włączonego sterownika.');
			return true;
		}

		if (empty($arg)){ //jeśli nie określono komendy dla IP
			$sbObject->AddAdminMessage('Podaj komendę dla IP (show, shownick, ban, unban, showbanned)');
			return true;
		}
		$ipCommands = explode(' ', $arg);
		$res = null;
		switch ($ipCommands[0]){ //wykonanie akcji dla zadanej komendy
			case 'show':
				$res = $this->show($ipCommands);
				break;
			case 'shownick':
				$res = $this->showNick($ipCommands);
				break;
			case 'ban':
				$res = $this->ban($ipCommands);
				break;
			case 'unban':
				$res = $this->unban($ipCommands);
				break;
			case 'showbanned':
				$res = $this->showBanned($ipCommands);
				break;
			default:
				$sbObject->AddAdminMessage('Nieobsługiwana komenda IP. Dostępne komendy to: show, shownick, ban, unban, showbanned');
				break;	
		}
		if (is_string($res)) 
			$sbObject->AddAdminMessage($res);
		return true;
	}
	
	/**
	 * Funkcja dla danego nicka zczytuje IP, z których pisał
	 *
	 * @param array $ipCommands
	 * @return string
	 */
	private function show($ipCommands){
		if (empty($ipCommands[1])){ //nie określono nicka, dla którego wyświetlić IP
			return 'Podaj nick, dla którego chcesz wyświetlić IP';
		}
		global $cfg_shoutbox;
		ShoutBoxDBMysql::MysqlConnect();
		$nick = mysql_escape_string($ipCommands[1]);
		$sql = "select IP, max(`TIME`) `TIME` from $cfg_shoutbox[table_name] where NICK='$nick' group by IP order by `TIME` desc limit 100"; //można by się pokusić o stronicowanie
		$res = mysql_query($sql);
		if ($res === false){
			return 'Błąd wykonywania zapytania: '.mysql_error();
		}
		if (!mysql_num_rows($res)){
			return 'Nie znaleziono żadnych adresów IP';
		}
		$ret = 'IP, z których pisał użytkownik:';
		while ($row = mysql_fetch_assoc($res)){
			if (!empty($row['IP']))
				$ret.='<br />'.$row['IP'].' - ostatni czas: '. $row['TIME'];
		}
		return $ret;
	}

	/**
	 * Funkcja dla danego IP wyświetla nicki, które pisały z tego IP
	 *
	 * @param array $ipCommands
	 * @return string
	 */
	private function showNick($ipCommands){
		if (empty($ipCommands[1])){ //nie określono IP, dla którego wyświetlić nicki
			return 'Podaj IP, dla którego chcesz wyświetlić nicki';
		}
		global $cfg_shoutbox;
		ShoutBoxDBMysql::MysqlConnect();
		$ip = $ipCommands[1];
		if (substr($ip,strlen($ip)-2) == '.*'){
			$ip = mysql_escape_string(str_replace('*','%',$ip));
			$where = "IP like '$ip'";
		} else {
			$ip = mysql_escape_string($ip);
			$where = "IP = '$ip'";
		}
		
		$sql = "select NICK, max(`TIME`) `TIME`, IP from $cfg_shoutbox[table_name] where $where group by NICK,IP order by `TIME` desc limit 100"; //można by się pokusić o stronicowanie
		$res = mysql_query($sql);
		if ($res === false){
			return 'Błąd wykonywania zapytania: '.mysql_error();
		}
		if (!mysql_num_rows($res)){
			return 'Nie znaleziono żadnych osób';
		}
		$ret = 'Osoby, które pisały z podanego IP:';
		while ($row = mysql_fetch_assoc($res)){
			if (!empty($row['NICK']))
				$ret.='<br />'.htmlspecialchars($row['NICK']).' '.$row['IP'].' - ostatni czas: '. $row['TIME'];
		}
		return $ret;
	}

	/**
	 * Funkcja banuje podane IP
	 * IP zapisuje do dwuwymiarowej tablicy. Tablia zawierać będzie indexy:
	 *  - full - będzie to tablica pełnych IP, np. 172.30.1.9, 192.168.20.4
	 *  - like - tablica z ogólnymi IP, np. 255.*, 123.12.*
	 *  Wartościami tablic będzie tekst, jaki zostanie wyświetlony osobie, próbującej dostać się do ShoutBoxa z zbanowanego IP
	 * @param array $ipCommands
	 * @return string
	 */
	private function ban($ipCommands){
		if (empty($ipCommands[1])){ //nie określono IP, które ma być zbanowane
			return 'Podaj IP, które chcesz zbanować';
		}
		
		if (file_exists('./sb_banned.php')){
			$banned = include('./sb_banned.php');
			if (!is_array($banned))
				$banned = array('full' => array(), 'like' => array());
		} else {
			$banned = array('full' => array(), 'like' => array());
		}
		
		$ip = $ipCommands[1];
		if (!empty($ipCommands[2])){ //podano jeszcze tekst banowania
			unset($ipCommands[0], $ipCommands[1]);//usuwamy komende i IP
			$text = implode(' ', $ipCommands);
		} else
			$text = '';
		if (substr($ip,strlen($ip)-2) == '.*')
			$banned['like'][$ip] = $text;
		else 
			$banned['full'][$ip] = $text;
		if ($this->saveBannedFile($banned))
			return 'IP dodano';
		else	
			return 'Nie udało się utworzyć pliku ze zbanowanymi IP';
	}

	/**
	 * Funkcja usuwa podane IP z banów
	 * @param array $ipCommands
	 * @return string
	 */
	private function unban($ipCommands){
		if (empty($ipCommands[1])){ //nie określono IP, które ma być zbanowane
			return 'Podaj IP, które chcesz odbanować';
		}
		
		if (!file_exists('./sb_banned.php'))
			return 'Nie istnieje plik z IP';
			
		$banned = include('./sb_banned.php');
		if (!is_array($banned))
			return 'Błąd odczytu pliku z IP';
		
		$ip = $ipCommands[1];	
		if (substr($ip,strlen($ip)-2) == '.*')
			$ind = 'like';
		else 
			$ind = 'full';
		if (isset($banned[$ind][$ip]))
			unset($banned[$ind][$ip]);
		else
			return 'Podane IP nie istnieje';		
		if (empty($banned['full']) && empty($banned['like'])){
			if (unlink('./sb_banned.php'))	
				return 'IP usunięto';
			else	
				return 'Nie udało się usunąć pliku z banami';
		}
		if ($this->saveBannedFile($banned))
			return 'IP usunięto';
		else	
			return 'Nie udało się utworzyć pliku ze zbanowanymi IP';
			
	}
	
	/**
	 * Tworzy plik php, jako tablice podanych IP
	 *
	 * @param array $banned
	 * @return boolean
	 */
	private function saveBannedFile($banned){
		$fileStr = "<?php\nreturn array(\n'full' => array(\n";
		if (!empty($banned['full'])){
			foreach ($banned['full'] as $ip => $text){
				$ip = htmlspecialchars($ip, ENT_QUOTES);
				$text = htmlspecialchars($text, ENT_QUOTES);
				$fileStr.= "'$ip' => '$text',\n";
			}
		}
		$fileStr.= "),\n'like' => array(\n";
		if (!empty($banned['like'])){
			foreach ($banned['like'] as $ip => $text){
				$ip = htmlspecialchars($ip, ENT_QUOTES);
				$text = htmlspecialchars($text, ENT_QUOTES);
				$fileStr.= "'$ip' => '$text',\n";
			}
		}
		$fileStr.= ")); ?>";
		
		return file_put_contents('./sb_banned.php', $fileStr);
	}
	
	/**
	 * Pokazuje zbanowane IP
	 * @param array $ipCommands
	 * @return string
	 */
	private function showBanned($ipCommands){
		if (!file_exists('./sb_banned.php'))
			return 'Nie istnieje plik z IP';
			
		$banned = include('./sb_banned.php');
		if (!is_array($banned))
			return 'Błąd odczytu pliku z IP';
		
		$retStr = '<br /><b>Pełne IP</b>:<br />';
		foreach ($banned['full'] as $ip => $text)
			$retStr.="$ip - $text<br />";	
		$retStr.= '<b>Ogólne IP</b>:<br />';
		foreach ($banned['like'] as $ip => $text)
			$retStr.="$ip - $text<br />";
		return $retStr;		
	}
	
}
?>